// created by jay 0.7 (c) 1998 Axel.Schreiner@informatik.uni-osnabrueck.de

#line 2 "Parser.jay"
using System.Text;
using System.IO;
using System.Collections;
using System;

namespace Mono.Debugger.Frontends.CommandLine
{
	public class Parser
	{
		string name;
		ScriptingContext current_context;
		MyTextReader reader;
		bool do_quit = false;

		protected bool yacc_verbose_flag = false;

		public bool yacc_verbose {
			set {
				yacc_verbose_flag = value;
			}

			get {
				return yacc_verbose_flag;
			}
		}

		public bool Quit {
			get {
				return do_quit;
			}
		}

		Tokenizer lexer;

		public Tokenizer Lexer {
			get {
				return lexer;
			}
		}		   

#line default

  /** simplified error message.
      @see <a href="#yyerror(java.lang.String, java.lang.String[])">yyerror</a>
    */
  public void yyerror (string message) {
    yyerror(message, null);
  }

  /** (syntax) error message.
      Can be overwritten to control message format.
      @param message text to be displayed.
      @param expected vector of acceptable tokens, if available.
    */
  public void yyerror (string message, string[] expected) {
    if ((expected != null) && (expected.Length  > 0)) {
      System.Console.Write (message+", expecting");
      for (int n = 0; n < expected.Length; ++ n)
        System.Console.Write (" "+expected[n]);
        System.Console.WriteLine ();
    } else
      System.Console.WriteLine (message);
  }

  /** debugging support, requires the package jay.yydebug.
      Set to null to suppress debugging messages.
    */
  protected yydebug.yyDebug debug;

  protected static  int yyFinal = 2;
  public static  string [] yyRule = {
    "$accept : command_loop",
    "command_loop :",
    "command_loop : command_loop command EOL",
    "command_loop : command_loop EOL",
    "command_loop : command_loop EOF",
    "command_loop : QUIT EOL",
    "command : generated_command",
    "command : other_command",
    "command : assert_command",
    "generated_command : CONTINUE process_expression",
    "generated_command : STEP process_expression",
    "generated_command : NEXT process_expression",
    "generated_command : STEPI process_expression",
    "generated_command : NEXTI process_expression",
    "generated_command : FINISH process_expression",
    "generated_command : BACKTRACE process_expression",
    "generated_command : UP process_expression",
    "generated_command : DOWN process_expression",
    "generated_command : KILL process_expression",
    "generated_command : SHOW PROCESSES",
    "generated_command : SHOW REGISTERS frame_expression",
    "generated_command : SHOW PARAMETERS frame_expression",
    "generated_command : SHOW LOCALS frame_expression",
    "generated_command : SHOW MODULES",
    "generated_command : SHOW SOURCES module_list_expression",
    "generated_command : SHOW METHODS source_list_expression",
    "generated_command : SHOW BREAKPOINTS",
    "generated_command : SHOW THREADGROUPS",
    "generated_command : SHOW TYPE variable_expression",
    "generated_command : BREAK thread_group_expression source_expression",
    "generated_command : BREAKPOINT ENABLE breakpoint_number_expression",
    "generated_command : BREAKPOINT DISABLE breakpoint_number_expression",
    "generated_command : BREAKPOINT DELETE breakpoint_number_expression",
    "generated_command : FRAME frame_expression",
    "generated_command : DISASSEMBLE frame_expression",
    "generated_command : DISASSEMBLE METHOD frame_expression",
    "generated_command : PROCESS process_expression",
    "generated_command : BACKGROUND process_expression",
    "generated_command : STOP process_expression",
    "generated_command : MODULE module_list_expression module_operations",
    "generated_command : THREADGROUP CREATE STRING",
    "generated_command : THREADGROUP ADD STRING process_list_expression",
    "generated_command : THREADGROUP REMOVE STRING process_list_expression",
    "generated_command : LIST source_expression",
    "generated_command : LIST CONTINUE",
    "other_command : SAVE STRING",
    "other_command : SET BANG IDENTIFIER ASSIGN variable_expression",
    "other_command : PRINT expression",
    "other_command : START program_arguments_expression",
    "kind : UNKNOWN",
    "kind : FUNDAMENTAL",
    "kind : ARRAY",
    "kind : STRUCT",
    "kind : CLASS",
    "kind : POINTER",
    "assert_command : ASSERT KIND kind variable_expression",
    "assert_command : ASSERT ACCESSIBLE variable_expression",
    "assert_command : ASSERT NOT ACCESSIBLE variable_expression",
    "assert_command : ASSERT TYPE STRING variable_expression",
    "assert_command : ASSERT CONTENTS STRING variable_expression",
    "assert_command : ASSERT LINE INTEGER",
    "expression : frame_expression PERCENT IDENTIFIER",
    "expression : LENGTH OPEN_PARENS variable_expression CLOSE_PARENS",
    "expression : LOWER OPEN_PARENS variable_expression CLOSE_PARENS",
    "expression : UPPER OPEN_PARENS variable_expression CLOSE_PARENS",
    "expression : NUMBER",
    "expression : INTEGER",
    "expression : expression binary_operator expression",
    "expression : OPEN_PARENS expression CLOSE_PARENS",
    "expression : variable_expression",
    "variable_expression : STAR variable_expression",
    "variable_expression : BANG IDENTIFIER",
    "variable_expression : frame_expression DOLLAR IDENTIFIER",
    "variable_expression : variable_expression DOT IDENTIFIER",
    "variable_expression : variable_expression OPEN_BRACKET expression CLOSE_BRACKET",
    "variable_expression : PARENT OPEN_PARENS variable_expression CLOSE_PARENS",
    "program_arguments_expression : program_arglist",
    "program_arglist : STRING",
    "program_arglist : program_arglist STRING",
    "breakpoint_number_expression : INTEGER",
    "thread_group_expression :",
    "thread_group_expression : OP_LT IDENTIFIER OP_GT",
    "thread_group_expression : OP_LT GLOBAL OP_GT",
    "process_expression :",
    "process_expression : AT INTEGER",
    "frame_expression : process_expression",
    "frame_expression : process_expression HASH INTEGER",
    "opt_line :",
    "opt_line : COLON INTEGER",
    "source_expression : frame_expression",
    "source_expression : frame_expression IDENTIFIER opt_line",
    "source_expression : frame_expression INTEGER",
    "source_expression : STRING",
    "source_expression : STRING COLON INTEGER",
    "source_expression : BANG INTEGER",
    "module_list_expression : number_list",
    "source_list_expression : number_list",
    "process_list_expression : number_list",
    "number_list : INTEGER",
    "number_list : number_list COMMA INTEGER",
    "module_operation : IGNORE",
    "module_operation : UNIGNORE",
    "module_operation : BANG IGNORE",
    "module_operation : STEP",
    "module_operation : BANG STEP",
    "module_operation : SHOW BREAKPOINTS",
    "module_operations : module_operation_list",
    "module_operation_list : module_operation",
    "module_operation_list : module_operation_list module_operation",
    "binary_operator : STAR",
    "binary_operator : PLUS",
    "binary_operator : MINUS",
    "binary_operator : DIV",
  };
  protected static  string [] yyName = {    
    "end-of-file",null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,"QUIT","EOL","EOF","NONE","ERROR",
    "IDENTIFIER","INTEGER","NUMBER","STRING","HASH","AT","PERCENT",
    "DOLLAR","DOT","BANG","COMMA","ASSIGN","STAR","PLUS","MINUS","DIV",
    "OPEN_PARENS","CLOSE_PARENS","OPEN_BRACKET","CLOSE_BRACKET","OP_LT",
    "OP_GT","COLON","PROCESSES","REGISTERS","PARAMETERS","LOCALS","TYPE",
    "METHOD","PARENT","LENGTH","LOWER","UPPER","KIND","ACCESSIBLE",
    "CONTENTS","LINE","NOT","UNKNOWN","FUNDAMENTAL","ARRAY","STRUCT",
    "CLASS","POINTER","MODULES","SOURCES","METHODS","BREAKPOINTS",
    "THREADGROUPS","GLOBAL","FRAME","START","SHOW","PROCESS","CONTINUE",
    "BACKTRACE","UP","DOWN","STEP","NEXT","STEPI","NEXTI","FINISH",
    "BREAK","PRINT","DISASSEMBLE","SET","ASSERT","BREAKPOINT","SAVE",
    "KILL","THREADGROUP","SEARCH","LIST","SELECT","BACKGROUND","STOP",
    "MODULE","IGNORE","UNIGNORE","ALL","ENABLE","DISABLE","DELETE",
    "CREATE","ADD","REMOVE","HELP",
  };

  /** index-checked interface to yyName[].
      @param token single character or %token value.
      @return token name or [illegal] or [unknown].
    */
  public static string yyname (int token) {
    if ((token < 0) || (token > yyName.Length)) return "[illegal]";
    string name;
    if ((name = yyName[token]) != null) return name;
    return "[unknown]";
  }

  /** computes list of expected tokens on error by tracing the tables.
      @param state for which to compute the list.
      @return list of token names.
    */
  protected string[] yyExpecting (int state) {
    int token, n, len = 0;
    bool[] ok = new bool[yyName.Length];

    if ((n = yySindex[state]) != 0)
      for (token = n < 0 ? -n : 0;
           (token < yyName.Length) && (n+token < yyTable.Length); ++ token)
        if (yyCheck[n+token] == token && !ok[token] && yyName[token] != null) {
          ++ len;
          ok[token] = true;
        }
    if ((n = yyRindex[state]) != 0)
      for (token = n < 0 ? -n : 0;
           (token < yyName.Length) && (n+token < yyTable.Length); ++ token)
        if (yyCheck[n+token] == token && !ok[token] && yyName[token] != null) {
          ++ len;
          ok[token] = true;
        }

    string [] result = new string[len];
    for (n = token = 0; n < len;  ++ token)
      if (ok[token]) result[n++] = yyName[token];
    return result;
  }

  /** the generated parser, with debugging messages.
      Maintains a state and a value stack, currently with fixed maximum size.
      @param yyLex scanner.
      @param yydebug debug message writer implementing yyDebug, or null.
      @return result of the last reduction, if any.
      @throws yyException on irrecoverable parse error.
    */
  public Object yyparse (yyParser.yyInput yyLex, Object yyd)
				 {
    this.debug = (yydebug.yyDebug)yyd;
    return yyparse(yyLex);
  }

  /** initial size and increment of the state/value stack [default 256].
      This is not final so that it can be overwritten outside of invocations
      of yyparse().
    */
  protected int yyMax;

  /** executed at the beginning of a reduce action.
      Used as $$ = yyDefault($1), prior to the user-specified action, if any.
      Can be overwritten to provide deep copy, etc.
      @param first value for $1, or null.
      @return first.
    */
  protected Object yyDefault (Object first) {
    return first;
  }

  /** the generated parser.
      Maintains a state and a value stack, currently with fixed maximum size.
      @param yyLex scanner.
      @return result of the last reduction, if any.
      @throws yyException on irrecoverable parse error.
    */
  public Object yyparse (yyParser.yyInput yyLex)
				{
    if (yyMax <= 0) yyMax = 256;			// initial size
    int yyState = 0;                                   // state stack ptr
    int [] yyStates = new int[yyMax];	                // state stack 
    Object yyVal = null;                               // value stack ptr
    Object [] yyVals = new Object[yyMax];	        // value stack
    int yyToken = -1;					// current input
    int yyErrorFlag = 0;				// #tks to shift

    int yyTop = 0;
    goto skip;
    yyLoop:
    yyTop++;
    skip:
    for (;; ++ yyTop) {
      if (yyTop >= yyStates.Length) {			// dynamically increase
        int[] i = new int[yyStates.Length+yyMax];
        yyStates.CopyTo (i, 0);
        yyStates = i;
        Object[] o = new Object[yyVals.Length+yyMax];
        yyVals.CopyTo (o, 0);
        yyVals = o;
      }
      yyStates[yyTop] = yyState;
      yyVals[yyTop] = yyVal;
      if (debug != null) debug.push(yyState, yyVal);

      yyDiscarded: for (;;) {	// discarding a token does not change stack
        int yyN;
        if ((yyN = yyDefRed[yyState]) == 0) {	// else [default] reduce (yyN)
          if (yyToken < 0) {
            yyToken = yyLex.advance() ? yyLex.token() : 0;
            if (debug != null)
              debug.lex(yyState, yyToken, yyname(yyToken), yyLex.value());
          }
          if ((yyN = yySindex[yyState]) != 0 && ((yyN += yyToken) >= 0)
              && (yyN < yyTable.Length) && (yyCheck[yyN] == yyToken)) {
            if (debug != null)
              debug.shift(yyState, yyTable[yyN], yyErrorFlag-1);
            yyState = yyTable[yyN];		// shift to yyN
            yyVal = yyLex.value();
            yyToken = -1;
            if (yyErrorFlag > 0) -- yyErrorFlag;
            goto yyLoop;
          }
          if ((yyN = yyRindex[yyState]) != 0 && (yyN += yyToken) >= 0
              && yyN < yyTable.Length && yyCheck[yyN] == yyToken)
            yyN = yyTable[yyN];			// reduce (yyN)
          else
            switch (yyErrorFlag) {
  
            case 0:
              yyerror("syntax error", yyExpecting(yyState));
              if (debug != null) debug.error("syntax error");
              goto case 1;
            case 1: case 2:
              yyErrorFlag = 3;
              do {
                if ((yyN = yySindex[yyStates[yyTop]]) != 0
                    && (yyN += Token.yyErrorCode) >= 0 && yyN < yyTable.Length
                    && yyCheck[yyN] == Token.yyErrorCode) {
                  if (debug != null)
                    debug.shift(yyStates[yyTop], yyTable[yyN], 3);
                  yyState = yyTable[yyN];
                  yyVal = yyLex.value();
                  goto yyLoop;
                }
                if (debug != null) debug.pop(yyStates[yyTop]);
              } while (-- yyTop >= 0);
              if (debug != null) debug.reject();
              throw new yyParser.yyException("irrecoverable syntax error");
  
            case 3:
              if (yyToken == 0) {
                if (debug != null) debug.reject();
                throw new yyParser.yyException("irrecoverable syntax error at end-of-file");
              }
              if (debug != null)
                debug.discard(yyState, yyToken, yyname(yyToken),
  							yyLex.value());
              yyToken = -1;
              goto yyDiscarded;		// leave stack alone
            }
        }
        int yyV = yyTop + 1-yyLen[yyN];
        if (debug != null)
          debug.reduce(yyState, yyStates[yyV-1], yyN, yyRule[yyN], yyLen[yyN]);
        yyVal = yyDefault(yyV > yyTop ? null : yyVals[yyV]);
        switch (yyN) {
case 2:
#line 152 "Parser.jay"
  {
		((Command) yyVals[-1+yyTop]).Execute (current_context);
	  }
  break;
case 5:
#line 158 "Parser.jay"
  {
		do_quit = true;
		reader.Close ();
	  }
  break;
case 9:
#line 168 "Parser.jay"
  {
		yyVal = new ContinueCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 10:
#line 172 "Parser.jay"
  {
		yyVal = new StepCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 11:
#line 176 "Parser.jay"
  {
		yyVal = new NextCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 12:
#line 180 "Parser.jay"
  {
		yyVal = new StepInstructionCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 13:
#line 184 "Parser.jay"
  {
		yyVal = new NextInstructionCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 14:
#line 188 "Parser.jay"
  {
		yyVal = new FinishCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 15:
#line 192 "Parser.jay"
  {
		yyVal = new BacktraceCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 16:
#line 196 "Parser.jay"
  {
		yyVal = new UpCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 17:
#line 200 "Parser.jay"
  {
		yyVal = new DownCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 18:
#line 204 "Parser.jay"
  {
		yyVal = new KillProcessCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 19:
#line 208 "Parser.jay"
  {
		yyVal = new ShowProcessesCommand ();
	  }
  break;
case 20:
#line 212 "Parser.jay"
  {
		yyVal = new ShowRegistersCommand ((FrameExpression) yyVals[0+yyTop]);
	  }
  break;
case 21:
#line 216 "Parser.jay"
  {
		yyVal = new ShowParametersCommand ((FrameExpression) yyVals[0+yyTop]);
	  }
  break;
case 22:
#line 220 "Parser.jay"
  {
		yyVal = new ShowLocalsCommand ((FrameExpression) yyVals[0+yyTop]);
	  }
  break;
case 23:
#line 224 "Parser.jay"
  {
		yyVal = new ShowModulesCommand ();
	  }
  break;
case 24:
#line 228 "Parser.jay"
  {
		yyVal = new ShowSourcesCommand ((ModuleListExpression) yyVals[0+yyTop]);
	  }
  break;
case 25:
#line 232 "Parser.jay"
  {
		yyVal = new ShowMethodsCommand ((SourceListExpression) yyVals[0+yyTop]);
	  }
  break;
case 26:
#line 236 "Parser.jay"
  {
		yyVal = new ShowBreakpointsCommand ();
	  }
  break;
case 27:
#line 240 "Parser.jay"
  {
		yyVal = new ShowThreadGroupsCommand ();
	  }
  break;
case 28:
#line 244 "Parser.jay"
  {
		yyVal = new ShowVariableTypeCommand ((VariableExpression) yyVals[0+yyTop]);
	  }
  break;
case 29:
#line 248 "Parser.jay"
  {
		yyVal = new BreakCommand ((ThreadGroupExpression) yyVals[-1+yyTop], (SourceExpression) yyVals[0+yyTop]);
	  }
  break;
case 30:
#line 252 "Parser.jay"
  {
		yyVal = new BreakpointEnableCommand ((BreakpointNumberExpression) yyVals[0+yyTop]);
	  }
  break;
case 31:
#line 256 "Parser.jay"
  {
		yyVal = new BreakpointDisableCommand ((BreakpointNumberExpression) yyVals[0+yyTop]);
	  }
  break;
case 32:
#line 260 "Parser.jay"
  {
		yyVal = new BreakpointDeleteCommand ((BreakpointNumberExpression) yyVals[0+yyTop]);
	  }
  break;
case 33:
#line 264 "Parser.jay"
  {
		yyVal = new FrameCommand ((FrameExpression) yyVals[0+yyTop]);
	  }
  break;
case 34:
#line 268 "Parser.jay"
  {
		yyVal = new DisassembleCommand ((FrameExpression) yyVals[0+yyTop]);
	  }
  break;
case 35:
#line 272 "Parser.jay"
  {
		yyVal = new DisassembleMethodCommand ((FrameExpression) yyVals[0+yyTop]);
	  }
  break;
case 36:
#line 276 "Parser.jay"
  {
		yyVal = new SelectProcessCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 37:
#line 280 "Parser.jay"
  {
		yyVal = new BackgroundProcessCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 38:
#line 284 "Parser.jay"
  {
		yyVal = new StopProcessCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 39:
#line 288 "Parser.jay"
  {
		yyVal = new ModuleOperationCommand ((ModuleListExpression) yyVals[-1+yyTop], (ModuleOperationListExpression) yyVals[0+yyTop]);
	  }
  break;
case 40:
#line 292 "Parser.jay"
  {
		yyVal = new ThreadGroupCreateCommand ((string) yyVals[0+yyTop]);
	  }
  break;
case 41:
#line 296 "Parser.jay"
  {
		yyVal = new ThreadGroupAddCommand ((string) yyVals[-1+yyTop], (ProcessListExpression) yyVals[0+yyTop]);
	  }
  break;
case 42:
#line 300 "Parser.jay"
  {
		yyVal = new ThreadGroupRemoveCommand ((string) yyVals[-1+yyTop], (ProcessListExpression) yyVals[0+yyTop]);
	  }
  break;
case 43:
#line 304 "Parser.jay"
  {
		yyVal = new ListCommand ((SourceExpression) yyVals[0+yyTop]);
	  }
  break;
case 44:
#line 308 "Parser.jay"
  {
		yyVal = new ListCommand (null);
	  }
  break;
case 45:
#line 315 "Parser.jay"
  {
		yyVal = new SaveCommand ((string) yyVals[0+yyTop]);
	  }
  break;
case 46:
#line 319 "Parser.jay"
  {
		yyVal = new ScriptingVariableAssignCommand ((string) yyVals[-2+yyTop], (VariableExpression) yyVals[0+yyTop]);
	  }
  break;
case 47:
#line 323 "Parser.jay"
  {
		yyVal = new PrintCommand ((Expression) yyVals[0+yyTop]);
	  }
  break;
case 48:
#line 327 "Parser.jay"
  {
		yyVal = new StartCommand ((ProgramArgumentsExpression) yyVals[0+yyTop]);
	  }
  break;
case 49:
#line 334 "Parser.jay"
  {
		yyVal = TargetObjectKind.Unknown;
	  }
  break;
case 50:
#line 338 "Parser.jay"
  {
		yyVal = TargetObjectKind.Fundamental;
	  }
  break;
case 51:
#line 342 "Parser.jay"
  {
		yyVal = TargetObjectKind.Array;
	  }
  break;
case 52:
#line 346 "Parser.jay"
  {
		yyVal = TargetObjectKind.Struct;
	  }
  break;
case 53:
#line 350 "Parser.jay"
  {
		yyVal = TargetObjectKind.Class;
	  }
  break;
case 54:
#line 354 "Parser.jay"
  {
		yyVal = TargetObjectKind.Pointer;
	  }
  break;
case 55:
#line 361 "Parser.jay"
  {
		yyVal = new AssertKindCommand ((TargetObjectKind) yyVals[-1+yyTop], (VariableExpression) yyVals[0+yyTop]);
	  }
  break;
case 56:
#line 365 "Parser.jay"
  {
		yyVal = new AssertAccessibleCommand ((VariableExpression) yyVals[0+yyTop], true);
	  }
  break;
case 57:
#line 369 "Parser.jay"
  {
		yyVal = new AssertAccessibleCommand ((VariableExpression) yyVals[0+yyTop], false);
	  }
  break;
case 58:
#line 373 "Parser.jay"
  {
		yyVal = new AssertTypeCommand ((string) yyVals[-1+yyTop], (VariableExpression) yyVals[0+yyTop]);
	  }
  break;
case 59:
#line 377 "Parser.jay"
  {
		yyVal = new AssertContentsCommand ((string) yyVals[-1+yyTop], (VariableExpression) yyVals[0+yyTop]);
	  }
  break;
case 60:
#line 381 "Parser.jay"
  {
		yyVal = new AssertLineCommand ((int) yyVals[0+yyTop]);
	  }
  break;
case 61:
#line 388 "Parser.jay"
  {
		yyVal = new RegisterExpression ((FrameExpression) yyVals[-2+yyTop], (string) yyVals[0+yyTop]);
	  }
  break;
case 62:
#line 392 "Parser.jay"
  {
		yyVal = new ArrayLengthExpression ((VariableExpression) yyVals[-1+yyTop]);
	  }
  break;
case 63:
#line 396 "Parser.jay"
  {
		yyVal = new ArrayLowerBoundExpression ((VariableExpression) yyVals[-1+yyTop]);
	  }
  break;
case 64:
#line 400 "Parser.jay"
  {
		yyVal = new ArrayUpperBoundExpression ((VariableExpression) yyVals[-1+yyTop]);
	  }
  break;
case 65:
#line 404 "Parser.jay"
  {
		yyVal = new NumberExpression ((int) yyVals[0+yyTop]);
	  }
  break;
case 66:
#line 408 "Parser.jay"
  {
		yyVal = new NumberExpression ((int) yyVals[0+yyTop]);
	  }
  break;
case 67:
#line 412 "Parser.jay"
  {
		yyVal = new BinaryOperator ((BinaryOperator.Kind) yyVals[-1+yyTop], (Expression) yyVals[-2+yyTop], (Expression) yyVals[0+yyTop]);
	  }
  break;
case 68:
#line 416 "Parser.jay"
  {
		yyVal = new ExpressionGroup ((Expression) yyVals[-1+yyTop]);
	  }
  break;
case 69:
#line 420 "Parser.jay"
  {
		yyVal = yyVals[0+yyTop];
	  }
  break;
case 70:
#line 427 "Parser.jay"
  {
		yyVal = new VariableDereferenceExpression ((VariableExpression) yyVals[0+yyTop]);
	  }
  break;
case 71:
#line 431 "Parser.jay"
  {
		yyVal = new ScriptingVariableReference ((string) yyVals[0+yyTop]);
	  }
  break;
case 72:
#line 435 "Parser.jay"
  {
		yyVal = new VariableReferenceExpression ((FrameExpression) yyVals[-2+yyTop], (string) yyVals[0+yyTop]);
	  }
  break;
case 73:
#line 439 "Parser.jay"
  {
		yyVal = new StructAccessExpression ((VariableExpression) yyVals[-2+yyTop], (string) yyVals[0+yyTop]);
	  }
  break;
case 74:
#line 443 "Parser.jay"
  {
		yyVal = new ArrayAccessExpression ((VariableExpression) yyVals[-3+yyTop], (Expression) yyVals[-1+yyTop]);
	  }
  break;
case 75:
#line 447 "Parser.jay"
  {
		yyVal = new ParentClassExpression ((VariableExpression) yyVals[-1+yyTop]);
	  }
  break;
case 76:
#line 454 "Parser.jay"
  {
		string[] args = new string [((ArrayList) yyVals[0+yyTop]).Count];
		((ArrayList) yyVals[0+yyTop]).CopyTo (args, 0);
		yyVal = new ProgramArgumentsExpression (args);
	  }
  break;
case 77:
#line 463 "Parser.jay"
  {
		ArrayList args = new ArrayList ();
		args.Add (yyVals[0+yyTop]);

		yyVal = args;
	  }
  break;
case 78:
#line 470 "Parser.jay"
  {
		ArrayList args = (ArrayList) yyVals[-1+yyTop];
		args.Add (yyVals[0+yyTop]);

		yyVal = args;
	  }
  break;
case 79:
#line 480 "Parser.jay"
  {
		yyVal = new BreakpointNumberExpression ((int) yyVals[0+yyTop]);
	  }
  break;
case 80:
#line 487 "Parser.jay"
  {
		yyVal = new ThreadGroupExpression ("main");
	  }
  break;
case 81:
#line 491 "Parser.jay"
  {
		yyVal = new ThreadGroupExpression ((string) yyVals[0+yyTop]);
	  }
  break;
case 82:
#line 495 "Parser.jay"
  {
		yyVal = new ThreadGroupExpression ("global");
	  }
  break;
case 83:
#line 502 "Parser.jay"
  {
		yyVal = new ProcessExpression (-1);
	  }
  break;
case 84:
#line 506 "Parser.jay"
  {
		yyVal = new ProcessExpression ((int) yyVals[0+yyTop]);
	  }
  break;
case 85:
#line 513 "Parser.jay"
  {
		yyVal = new FrameExpression ((ProcessExpression) yyVals[0+yyTop], -1);
	  }
  break;
case 86:
#line 517 "Parser.jay"
  {
		yyVal = new FrameExpression ((ProcessExpression) yyVals[-2+yyTop], (int) yyVals[0+yyTop]);
	  }
  break;
case 87:
#line 524 "Parser.jay"
  {
		yyVal = -1;
	  }
  break;
case 88:
#line 528 "Parser.jay"
  {
		yyVal = yyVals[0+yyTop];
	  }
  break;
case 89:
#line 535 "Parser.jay"
  {
		yyVal = new SourceExpression ((FrameExpression) yyVals[0+yyTop], null, -1);
	  }
  break;
case 90:
#line 539 "Parser.jay"
  {
		yyVal = new SourceExpression ((FrameExpression) yyVals[-2+yyTop], (string) yyVals[-1+yyTop], (int) yyVals[0+yyTop]);
	  }
  break;
case 91:
#line 543 "Parser.jay"
  {
		yyVal = new SourceExpression ((FrameExpression) yyVals[-1+yyTop], (int) yyVals[0+yyTop]);
	  }
  break;
case 92:
#line 547 "Parser.jay"
  {
		yyVal = new SourceExpression ((string) yyVals[0+yyTop]);
	  }
  break;
case 93:
#line 551 "Parser.jay"
  {
		yyVal = new SourceExpression ((string) yyVals[-2+yyTop], (int) yyVals[0+yyTop]);
	  }
  break;
case 94:
#line 555 "Parser.jay"
  {
		yyVal = new SourceExpression ((int) yyVals[0+yyTop]);
	  }
  break;
case 95:
#line 562 "Parser.jay"
  {
		int[] indices = new int [((ArrayList) yyVals[0+yyTop]).Count];
		((ArrayList) yyVals[0+yyTop]).CopyTo (indices, 0);

		yyVal = new ModuleListExpression (indices);
	  }
  break;
case 96:
#line 572 "Parser.jay"
  {
		int[] indices = new int [((ArrayList) yyVals[0+yyTop]).Count];
		((ArrayList) yyVals[0+yyTop]).CopyTo (indices, 0);

		yyVal = new SourceListExpression (indices);
	  }
  break;
case 97:
#line 582 "Parser.jay"
  {
		int[] indices = new int [((ArrayList) yyVals[0+yyTop]).Count];
		((ArrayList) yyVals[0+yyTop]).CopyTo (indices, 0);

		yyVal = new ProcessListExpression (indices);
	  }
  break;
case 98:
#line 592 "Parser.jay"
  {
		ArrayList args = new ArrayList ();
		args.Add (yyVals[0+yyTop]);

		yyVal = args;
	  }
  break;
case 99:
#line 599 "Parser.jay"
  {
		ArrayList args = (ArrayList) yyVals[-2+yyTop];
		args.Add (yyVals[0+yyTop]);

		yyVal = args;
	  }
  break;
case 100:
#line 609 "Parser.jay"
  {
		yyVal = ModuleOperation.Ignore;
	  }
  break;
case 101:
#line 613 "Parser.jay"
  {
		yyVal = ModuleOperation.UnIgnore;
	  }
  break;
case 102:
#line 617 "Parser.jay"
  {
		yyVal = ModuleOperation.UnIgnore;
	  }
  break;
case 103:
#line 621 "Parser.jay"
  {
		yyVal = ModuleOperation.Step;
	  }
  break;
case 104:
#line 625 "Parser.jay"
  {
		yyVal = ModuleOperation.DontStep;
	  }
  break;
case 105:
#line 629 "Parser.jay"
  {
		yyVal = ModuleOperation.ShowBreakpoints;
	  }
  break;
case 106:
#line 636 "Parser.jay"
  {
		ModuleOperation[] operations = new ModuleOperation [((ArrayList) yyVals[0+yyTop]).Count];
		((ArrayList) yyVals[0+yyTop]).CopyTo (operations, 0);

		yyVal = new ModuleOperationListExpression (operations);
	  }
  break;
case 107:
#line 646 "Parser.jay"
  {
		ArrayList args = new ArrayList ();
		args.Add (yyVals[0+yyTop]);

		yyVal = args;
	  }
  break;
case 108:
#line 653 "Parser.jay"
  {
		ArrayList args = (ArrayList) yyVals[-1+yyTop];
		args.Add (yyVals[0+yyTop]);

		yyVal = args;
	  }
  break;
case 109:
#line 663 "Parser.jay"
  {
		yyVal = BinaryOperator.Kind.Mult;
	  }
  break;
case 110:
#line 667 "Parser.jay"
  {
		yyVal = BinaryOperator.Kind.Plus;
	  }
  break;
case 111:
#line 671 "Parser.jay"
  {
		yyVal = BinaryOperator.Kind.Minus;
	  }
  break;
case 112:
#line 675 "Parser.jay"
  {
		yyVal = BinaryOperator.Kind.Div;
	  }
  break;
#line default
        }
        yyTop -= yyLen[yyN];
        yyState = yyStates[yyTop];
        int yyM = yyLhs[yyN];
        if (yyState == 0 && yyM == 0) {
          if (debug != null) debug.shift(0, yyFinal);
          yyState = yyFinal;
          if (yyToken < 0) {
            yyToken = yyLex.advance() ? yyLex.token() : 0;
            if (debug != null)
               debug.lex(yyState, yyToken,yyname(yyToken), yyLex.value());
          }
          if (yyToken == 0) {
            if (debug != null) debug.accept(yyVal);
            return yyVal;
          }
          goto yyLoop;
        }
        if (((yyN = yyGindex[yyM]) != 0) && ((yyN += yyState) >= 0)
            && (yyN < yyTable.Length) && (yyCheck[yyN] == yyState))
          yyState = yyTable[yyN];
        else
          yyState = yyDgoto[yyM];
        if (debug != null) debug.shift(yyStates[yyTop], yyState);
	 goto yyLoop;
      }
    }
  }

   static  short [] yyLhs  = {              -1,
    0,    0,    0,    0,    0,    1,    1,    1,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    3,    3,    3,    3,   17,   17,
   17,   17,   17,   17,    4,    4,    4,    4,    4,    4,
   15,   15,   15,   15,   15,   15,   15,   15,   15,    9,
    9,    9,    9,    9,    9,   16,   19,   19,   12,   10,
   10,   10,    5,    5,    6,    6,   20,   20,   11,   11,
   11,   11,   11,   11,    7,    8,   14,   21,   21,   22,
   22,   22,   22,   22,   22,   13,   23,   23,   18,   18,
   18,   18,
  };
   static  short [] yyLen = {           2,
    0,    3,    2,    2,    2,    1,    1,    1,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    3,
    3,    3,    2,    3,    3,    2,    2,    3,    3,    3,
    3,    3,    2,    2,    3,    2,    2,    2,    3,    3,
    4,    4,    2,    2,    2,    5,    2,    2,    1,    1,
    1,    1,    1,    1,    4,    3,    4,    4,    4,    3,
    3,    4,    4,    4,    1,    1,    3,    3,    1,    2,
    2,    3,    3,    4,    4,    1,    1,    2,    1,    0,
    3,    3,    0,    2,    1,    3,    0,    2,    1,    3,
    2,    1,    3,    2,    1,    1,    1,    1,    3,    1,
    1,    2,    1,    2,    2,    1,    1,    2,    1,    1,
    1,    1,
  };
   static  short [] yyDefRed = {            0,
    0,    0,    5,    3,    4,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    6,    7,    8,    0,    0,   33,   77,   48,
    0,   19,    0,    0,    0,    0,   23,    0,    0,   26,
   27,   36,    9,   15,   16,   17,   10,   11,   12,   13,
   14,    0,    0,   66,   65,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   34,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   45,   18,    0,
    0,    0,    0,    0,   44,    0,   43,   37,   38,   98,
    0,    0,    2,   84,    0,   78,   20,   21,   22,    0,
    0,   24,   25,    0,    0,    0,   29,   71,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  109,  110,
  111,  112,    0,   35,    0,    0,   49,   50,   51,   52,
   53,   54,    0,    0,    0,   60,    0,   79,   30,   31,
   32,   40,    0,    0,    0,   94,    0,   91,    0,    0,
  103,  100,  101,   39,  107,    0,    0,   86,   81,   82,
   68,    0,    0,    0,    0,   61,   72,   73,    0,    0,
    0,    0,    0,    0,    0,   41,    0,   42,   93,    0,
   90,  104,  102,  105,  108,   99,   75,   62,   63,   64,
   74,    0,   88,
  };
  protected static  short [] yyDgoto  = {             2,
   32,   33,   34,   35,   37,  110,  101,  113,   74,   63,
   97,  149,  164,  186,   75,   40,  143,  133,   41,  191,
  102,  165,  166,
  };
  protected static  short [] yySindex = {         -195,
 -192, -176,    0,    0,    0, -196, -191, -194, -196, -196,
 -196, -196, -196, -196, -196, -196, -196, -196, -203, -244,
 -258, -182,  -91, -330, -169, -196, -262, -259, -196, -196,
 -165, -155,    0,    0,    0, -130,  -94,    0,    0,    0,
  -99,    0, -196, -196, -196, -245,    0, -165, -165,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, -255, -137,    0,    0,  -84, -245, -244,  -86,  -77,
  -69,  -51, -162, -227,  -52, -196,    0,  -82,  -68,  -90,
 -245,  -33,  -43,  -63,  -29,  -29,  -29,    0,    0,  -30,
  -28,  -27,  -48,  -24,    0,  -92,    0,    0,    0,    0,
 -260,  -32,    0,    0,  -22,    0,    0,    0,    0,  -26,
 -227,    0,    0,  -32,  -41,  -39,    0,    0, -227,  -58,
 -245, -245, -245, -245,  -17,  -16,  -15, -244,    0,    0,
    0,    0, -244,    0,  -25, -245,    0,    0,    0,    0,
    0,    0, -245, -227, -245,    0, -245,    0,    0,    0,
    0,    0, -165, -165,  -14,    0,  -34,    0, -295,  -57,
    0,    0,    0,    0,    0, -260,  -12,    0,    0,    0,
    0, -242, -228, -153, -148,    0,    0,    0,  -81,  -52,
 -245, -227, -227, -227, -227,    0,  -32,    0,    0,  -10,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, -227,    0,
  };
  protected static  short [] yyRindex = {            1,
    0,    0,    0,    0,    0, -222,    0,    0,   -3,   -3,
   -3,   -3,   -3,   -3,   -3,   -3,   -3,   -3,  -98,  -40,
 -222,    0,    0,    0,    0,   -3,    0, -193,   -3,   -3,
    0,    0,    0,    0,    0,    0, -158,    0,    0,    0,
   -2,    0, -222, -222, -222, -211,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, -193,    0,    0,    0, -211,  -40,    0,    0,
    0,    0,    0, -156,   -1, -222,    0,    0,    0,    0,
 -211,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    3,    0,    0,    4,    0,    0,    0,    0,
    0, -253,    0,    0,    0,    0,    0,    0,    0,    0,
    5,    0,    0,    6,    0,    0,    0,    0, -100,    0,
 -211, -211, -211, -211,    0,    0,    0,  -40,    0,    0,
    0,    0,  -40,    0,    0, -211,    0,    0,    0,    0,
    0,    0, -211,    7, -211,    0, -211,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    8,    0,    0,    0,
    0,    0,    0,    0,    0,    9,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, -248,
 -211,   10,   11,   12,   13,    0,   14,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   15,    0,
  };
  protected static  short [] yyGindex = {            0,
    0,    0,    0,    0,  173,   -4,  206,    0,  -46,    0,
  195,  144,    0,  120,  -65,    0,    0,    0,    0,    0,
  -45,  109,    0,
  };
  protected static  short [] yyTable = {           111,
    1,   38,  120,  114,   95,   93,  115,   36,   36,   67,
  159,   94,   85,   86,   87,   73,   77,   95,   64,   65,
  119,   36,   36,   96,  192,   66,   66,  127,   67,   67,
   67,   76,   67,   68,  144,   83,  197,  128,  107,  108,
  109,  127,  127,   83,  193,   69,   69,   70,   71,   72,
  198,  128,  128,  160,   83,  116,   95,   83,   96,  161,
   95,    1,  179,   73,   83,    3,   95,  180,   83,   83,
   36,  134,   83,   39,  172,  173,  174,  175,   62,  162,
  163,    4,    5,   90,   91,   92,   95,   95,   78,  182,
   42,   43,   44,   45,   46,   88,  183,  100,  184,   85,
  185,   69,  103,   85,   85,  125,  126,  187,  187,   85,
   85,   47,   48,   49,   50,   51,  127,   69,   69,   69,
   69,  127,   69,   73,   69,  199,  128,   93,   73,   36,
  200,  128,  104,   94,  202,    6,    7,    8,    9,   10,
   11,   12,   13,   14,   15,   16,   17,   18,   19,   20,
   21,   22,   23,   24,   25,   26,   27,   70,   28,   80,
   29,   30,   31,   80,   80,  106,   80,   80,   80,  157,
  158,  105,   80,   70,   70,   70,   70,  118,   70,  135,
   70,   52,   53,   54,   55,   56,   57,   58,   59,   60,
   61,  121,  129,  130,  131,  132,  136,   79,   89,  201,
  122,   98,   99,   80,   81,   82,   83,   84,  123,  137,
  138,  139,  140,  141,  142,  129,  130,  131,  132,  146,
  171,  129,  130,  131,  132,   83,  124,   83,   83,  150,
  151,  145,  147,  148,  152,  155,  153,  154,  156,  167,
  168,  169,  126,  170,  176,  177,  178,  181,  189,  190,
  196,  194,  203,  112,   83,   76,   47,  117,    1,    1,
   92,   89,   28,   96,   56,   87,  106,   58,   55,   59,
   57,   97,   46,  188,  195,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    0,    1,    0,    1,    1,    1,
  };
  protected static  short [] yyCheck = {            46,
    0,    6,   68,   49,  258,  265,  262,  267,  267,  258,
  271,  271,  343,  344,  345,   20,   21,  271,  263,  264,
   67,  267,  267,   28,  320,  271,  271,  270,  274,  274,
  279,  290,  281,  278,   81,  258,  279,  280,   43,   44,
   45,  270,  270,  266,  340,  291,  291,  292,  293,  294,
  279,  280,  280,  314,  266,  311,  316,  269,   63,  320,
  314,  257,  128,   68,  258,  258,  320,  133,  262,  263,
  267,   76,  266,  265,  121,  122,  123,  124,  282,  340,
  341,  258,  259,  346,  347,  348,  340,  341,  271,  136,
  285,  286,  287,  288,  289,  265,  143,  263,  145,  258,
  147,  258,  258,  262,  263,  268,  269,  153,  154,  268,
  269,  306,  307,  308,  309,  310,  270,  274,  275,  276,
  277,  270,  279,  128,  281,  279,  280,  265,  133,  267,
  279,  280,  263,  271,  181,  312,  313,  314,  315,  316,
  317,  318,  319,  320,  321,  322,  323,  324,  325,  326,
  327,  328,  329,  330,  331,  332,  333,  258,  335,  258,
  337,  338,  339,  262,  263,  265,  265,  266,  267,  262,
  263,  266,  271,  274,  275,  276,  277,  262,  279,  262,
  281,    9,   10,   11,   12,   13,   14,   15,   16,   17,
   18,  278,  274,  275,  276,  277,  265,  289,   26,  281,
  278,   29,   30,  295,  296,  297,  298,  299,  278,  300,
  301,  302,  303,  304,  305,  274,  275,  276,  277,  263,
  279,  274,  275,  276,  277,  266,  278,  268,  269,   86,
   87,  265,  296,  263,  265,  284,  265,  265,  263,  272,
  263,  283,  269,  283,  262,  262,  262,  273,  263,  284,
  263,  309,  263,   48,  258,  258,  258,   63,  258,  259,
  258,  258,  258,  258,  258,  258,  258,  258,  258,  258,
  258,  258,  258,  154,  166,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  312,  313,  314,  315,  316,  317,  318,  319,
  320,  321,  322,  323,  324,  325,  326,  327,  328,  329,
  330,  331,  332,  333,   -1,  335,   -1,  337,  338,  339,
  };

#line 680 "Parser.jay"

public Parser (ScriptingContext context, string name)
{
	this.name = name;
	this.reader = new MyTextReader ();
	this.current_context = context;

	lexer = new Tokenizer (context, reader, name);
}

public bool Parse (string text)
{
	try {
		reader.Text = text;
		lexer.restart ();
		if (yacc_verbose_flag)
			yyparse (lexer, new yydebug.yyDebugSimple ());
		else
			yyparse (lexer);
		return true;
	} catch (Exception e){
		// Please do not remove this, it is used during debugging
		// of the grammar
		//
		current_context.Error (lexer.location + "  : Parsing error ");
		current_context.Error (e.ToString ());
		return false;
	}
}

/* end end end */
}
#line default
namespace yydebug {
        using System;
	 public interface yyDebug {
		 void push (int state, Object value);
		 void lex (int state, int token, string name, Object value);
		 void shift (int from, int to, int errorFlag);
		 void pop (int state);
		 void discard (int state, int token, string name, Object value);
		 void reduce (int from, int to, int rule, string text, int len);
		 void shift (int from, int to);
		 void accept (Object value);
		 void error (string message);
		 void reject ();
	 }
	 
	 class yyDebugSimple : yyDebug {
		 void println (string s){
			 Console.WriteLine (s);
		 }
		 
		 public void push (int state, Object value) {
			 println ("push\tstate "+state+"\tvalue "+value);
		 }
		 
		 public void lex (int state, int token, string name, Object value) {
			 println("lex\tstate "+state+"\treading "+name+"\tvalue "+value);
		 }
		 
		 public void shift (int from, int to, int errorFlag) {
			 switch (errorFlag) {
			 default:				// normally
				 println("shift\tfrom state "+from+" to "+to);
				 break;
			 case 0: case 1: case 2:		// in error recovery
				 println("shift\tfrom state "+from+" to "+to
					     +"\t"+errorFlag+" left to recover");
				 break;
			 case 3:				// normally
				 println("shift\tfrom state "+from+" to "+to+"\ton error");
				 break;
			 }
		 }
		 
		 public void pop (int state) {
			 println("pop\tstate "+state+"\ton error");
		 }
		 
		 public void discard (int state, int token, string name, Object value) {
			 println("discard\tstate "+state+"\ttoken "+name+"\tvalue "+value);
		 }
		 
		 public void reduce (int from, int to, int rule, string text, int len) {
			 println("reduce\tstate "+from+"\tuncover "+to
				     +"\trule ("+rule+") "+text);
		 }
		 
		 public void shift (int from, int to) {
			 println("goto\tfrom state "+from+" to "+to);
		 }
		 
		 public void accept (Object value) {
			 println("accept\tvalue "+value);
		 }
		 
		 public void error (string message) {
			 println("error\t"+message);
		 }
		 
		 public void reject () {
			 println("reject");
		 }
		 
	 }
}
// %token constants
 class Token {
  public const int QUIT = 257;
  public const int EOL = 258;
  public const int EOF = 259;
  public const int NONE = 260;
  public const int ERROR = 261;
  public const int IDENTIFIER = 262;
  public const int INTEGER = 263;
  public const int NUMBER = 264;
  public const int STRING = 265;
  public const int HASH = 266;
  public const int AT = 267;
  public const int PERCENT = 268;
  public const int DOLLAR = 269;
  public const int DOT = 270;
  public const int BANG = 271;
  public const int COMMA = 272;
  public const int ASSIGN = 273;
  public const int STAR = 274;
  public const int PLUS = 275;
  public const int MINUS = 276;
  public const int DIV = 277;
  public const int OPEN_PARENS = 278;
  public const int CLOSE_PARENS = 279;
  public const int OPEN_BRACKET = 280;
  public const int CLOSE_BRACKET = 281;
  public const int OP_LT = 282;
  public const int OP_GT = 283;
  public const int COLON = 284;
  public const int PROCESSES = 285;
  public const int REGISTERS = 286;
  public const int PARAMETERS = 287;
  public const int LOCALS = 288;
  public const int TYPE = 289;
  public const int METHOD = 290;
  public const int PARENT = 291;
  public const int LENGTH = 292;
  public const int LOWER = 293;
  public const int UPPER = 294;
  public const int KIND = 295;
  public const int ACCESSIBLE = 296;
  public const int CONTENTS = 297;
  public const int LINE = 298;
  public const int NOT = 299;
  public const int UNKNOWN = 300;
  public const int FUNDAMENTAL = 301;
  public const int ARRAY = 302;
  public const int STRUCT = 303;
  public const int CLASS = 304;
  public const int POINTER = 305;
  public const int MODULES = 306;
  public const int SOURCES = 307;
  public const int METHODS = 308;
  public const int BREAKPOINTS = 309;
  public const int THREADGROUPS = 310;
  public const int GLOBAL = 311;
  public const int FRAME = 312;
  public const int START = 313;
  public const int SHOW = 314;
  public const int PROCESS = 315;
  public const int CONTINUE = 316;
  public const int BACKTRACE = 317;
  public const int UP = 318;
  public const int DOWN = 319;
  public const int STEP = 320;
  public const int NEXT = 321;
  public const int STEPI = 322;
  public const int NEXTI = 323;
  public const int FINISH = 324;
  public const int BREAK = 325;
  public const int PRINT = 326;
  public const int DISASSEMBLE = 327;
  public const int SET = 328;
  public const int ASSERT = 329;
  public const int BREAKPOINT = 330;
  public const int SAVE = 331;
  public const int KILL = 332;
  public const int THREADGROUP = 333;
  public const int SEARCH = 334;
  public const int LIST = 335;
  public const int SELECT = 336;
  public const int BACKGROUND = 337;
  public const int STOP = 338;
  public const int MODULE = 339;
  public const int IGNORE = 340;
  public const int UNIGNORE = 341;
  public const int ALL = 342;
  public const int ENABLE = 343;
  public const int DISABLE = 344;
  public const int DELETE = 345;
  public const int CREATE = 346;
  public const int ADD = 347;
  public const int REMOVE = 348;
  public const int HELP = 349;
  public const int yyErrorCode = 256;
 }
 namespace yyParser {
  using System;
  /** thrown for irrecoverable syntax errors and stack overflow.
    */
  public class yyException : System.Exception {
    public yyException (string message) : base (message) {
    }
  }

  /** must be implemented by a scanner object to supply input to the parser.
    */
  public interface yyInput {
    /** move on to next token.
        @return false if positioned beyond tokens.
        @throws IOException on input error.
      */
    bool advance (); // throws java.io.IOException;
    /** classifies current token.
        Should not be called if advance() returned false.
        @return current %token or single character.
      */
    int token ();
    /** associated with current token.
        Should not be called if advance() returned false.
        @return value for token().
      */
    Object value ();
  }
 }
} // close outermost namespace, that MUST HAVE BEEN opened in the prolog
